#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.views.customers`` -- Customer Views
"""

from sqlalchemy import and_

import edbob
from edbob.pyramid.views import SearchableAlchemyGridView, CrudView
from edbob.pyramid.forms import EnumFieldRenderer

import rattail


class CustomersGrid(SearchableAlchemyGridView):

    mapped_class = rattail.Customer
    config_prefix = 'customers'
    sort = 'name'
    clickable = True

    def join_map(self):
        return {
            'email':
                lambda q: q.outerjoin(rattail.CustomerEmailAddress, and_(
                    rattail.CustomerEmailAddress.parent_uuid == rattail.Customer.uuid,
                    rattail.CustomerEmailAddress.preference == 1)),
            'phone':
                lambda q: q.outerjoin(rattail.CustomerPhoneNumber, and_(
                    rattail.CustomerPhoneNumber.parent_uuid == rattail.Customer.uuid,
                    rattail.CustomerPhoneNumber.preference == 1)),
            }

    def filter_map(self):
        return self.make_filter_map(
            exact=['id'],
            ilike=['name'],
            email=self.filter_ilike(rattail.CustomerEmailAddress.address),
            phone=self.filter_ilike(rattail.CustomerPhoneNumber.number))

    def filter_config(self):
        return self.make_filter_config(
            include_filter_name=True,
            filter_type_name='lk',
            filter_label_phone="Phone Number",
            filter_label_email="Email Address",
            filter_label_id="ID")

    def sort_map(self):
        return self.make_sort_map(
            'id', 'name',
            email=self.sorter(rattail.CustomerEmailAddress.address),
            phone=self.sorter(rattail.CustomerPhoneNumber.number))

    def grid(self):
        g = self.make_grid()
        g.configure(
            include=[
                g.id.label("ID"),
                g.name,
                g.phone.label("Phone Number"),
                g.email.label("Email Address"),
                ],
            readonly=True)
        g.click_route_name = 'customer.read'
        return g


class CustomerCrud(CrudView):

    mapped_class = rattail.Customer
    home_route = 'customers'

    def fieldset(self, model):
        fs = self.make_fieldset(model)
        fs.email_preference.set(renderer=EnumFieldRenderer(edbob.EMAIL_PREFERENCE))
        fs.configure(
            include=[
                fs.id.label("ID"),
                fs.name,
                fs.phone.label("Phone Number"),
                fs.email.label("Email Address"),
                fs.email_preference,
                ])
        return fs


def includeme(config):

    config.add_route('customers', '/customers')
    config.add_view(CustomersGrid, route_name='customers',
                    renderer='/customers/index.mako',
                    permission='customers.list')

    config.add_route('customer.read', '/customers/{uuid}')
    config.add_view(CustomerCrud, attr='read', route_name='customer.read',
                    renderer='/customers/read.mako',
                    permission='customers.read')
